//+------------------------------------------------------------------+
//|                                                  Custom MACD.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2004, MetaQuotes Software Corp."
#property  link      "http://www.metaquotes.net/"
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 4
#property  indicator_color1  Silver
#property  indicator_color2  Yellow
#property  indicator_color3  DodgerBlue
#property  indicator_color4  Red
#property  indicator_width1  4
#property  indicator_width3  4
#property  indicator_width4  4
//---- indicator parameters
extern int FastEMA    = 12;
extern int SlowEMA    = 26;
extern int SignalSMA  = 9;
//---- indicator buffers
double     MacdBuffer[];
double     SignalBuffer[];
double     MacdLong[];
double     MacdShort[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexDrawBegin(1,SignalSMA);
   IndicatorDigits(Digits+1);
//---- indicator buffers mapping
   SetIndexBuffer(0,MacdBuffer);
   SetIndexBuffer(1,SignalBuffer);
   SetIndexBuffer(2,MacdLong);
   SetIndexBuffer(3,MacdShort);
   
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD("+FastEMA+","+SlowEMA+","+SignalSMA+")");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(2,"MACD Long" ); 
   SetIndexLabel(3,"MACD Short" );    
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   int counted_bars=IndicatorCounted();
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(int i=0; i<limit; i++)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++) {
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_SMA,i);
      if(MacdBuffer[i] > 0 && MacdBuffer[i] > SignalBuffer[i]) {
         MacdLong[i] = MacdBuffer[i];
         MacdShort[i] = 0; 
         MacdBuffer[i] = 0; 
      }
      else if(MacdBuffer[i] < 0 && MacdBuffer[i] < SignalBuffer[i]) {
         MacdShort[i] = MacdBuffer[i];
         MacdLong[i] = 0; 
         MacdBuffer[i] = 0; 
      } 
   }
//---- done
   return(0);
  }
//+------------------------------------------------------------------+